﻿// (c) Copyright Telerik Corp. 
// This source is subject to the Microsoft Public License. 
// See http://www.microsoft.com/opensource/licenses.mspx#Ms-PL. 
// All other rights reserved.

namespace Telerik.Web.Mvc
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.Specialized;
    using System.Diagnostics;
    using System.IO;
    using System.Text;
    using System.Web;
    using System.Web.Mvc;
    using System.Web.Routing;
    using System.Web.UI;
    using Moq;


    class ValueProvider : IValueProvider
    {
        private readonly IDictionary<string, ValueProviderResult> data;

        public ValueProvider(IDictionary<string, ValueProviderResult> data)
        {
            this.data = data;
        }

        public bool ContainsPrefix(string prefix)
        {
            throw new NotImplementedException();
        }

        public ValueProviderResult GetValue(string key)
        {
            ValueProviderResult result;
            bool found = data.TryGetValue(key, out result);
            if (found)
            {
                return result;
            }
            
            return null;
        }

    }

    public class ControllerTestDouble : Controller
    {
        public ControllerTestDouble(IDictionary<string, ValueProviderResult> valueProviderData, ViewDataDictionary viewData)
        {
            ValueProvider = new ValueProvider(valueProviderData);
            ViewData = viewData;
            ControllerContext = new ControllerContext(TestHelper.CreateRequestContext(), this);
        }
    }


    public static class TestHelper
    {
        public const string AppPathModifier = "/$(SESSION)";
        public const string ApplicationPath = "/app/";

        [DebuggerStepThrough]
        public static HtmlHelper CreateHtmlHelper()
        {
            var viewDataContainer = new Mock<IViewDataContainer>();

            viewDataContainer.SetupGet(container => container.ViewData).Returns(new ViewDataDictionary());

            ViewContext viewContext = CreateViewContext();

            var helper = new HtmlHelper(viewContext, viewDataContainer.Object);

            return helper;
        }

#if MVC2 || MVC3
        [DebuggerStepThrough]
        public static HtmlHelper<TModel> CreateHtmlHelper<TModel>() where TModel : class, new()
        {
            var viewDataContainer = new Mock<IViewDataContainer>();

            viewDataContainer.SetupGet(container => container.ViewData).Returns(new ViewDataDictionary { Model = new TModel() });

            ViewContext viewContext = CreateViewContext();

            var helper = new HtmlHelper<TModel>(viewContext, viewDataContainer.Object);

            return helper;
        }
#endif
        private static Mock<HttpContextBase> _httpContext;
        private static readonly object Sync = new object();

        [DebuggerStepThrough]
        public static Mock<HttpContextBase> CreateMockedHttpContext(bool createNew)
        {
            if (createNew)
            {
                return MockedHttpContext();
            }

            if (_httpContext == null)
            {
                lock (Sync)
                {
                    if (_httpContext == null)
                    {
                        _httpContext = MockedHttpContext();
                    }
                }
            }

            _httpContext.Setup(context => context.Items).Returns(new Hashtable());

            return _httpContext;
        }

        [DebuggerStepThrough]
        public static Mock<HttpContextBase> CreateMockedHttpContext()
        {
            return CreateMockedHttpContext(false);
        }

        private static Mock<HttpContextBase> MockedHttpContext()
        {
            var result = new Mock<HttpContextBase>();
            result.Setup(context => context.Server).Returns(new Mock<HttpServerUtilityBase>().Object);
            result.Setup(context => context.Request.AppRelativeCurrentExecutionFilePath).Returns("~/");
            result.Setup(context => context.Request.ApplicationPath).Returns(ApplicationPath);
            result.Setup(context => context.Request.Url).Returns(new Uri("http://localhost"));
            result.Setup(context => context.Request.PathInfo).Returns(string.Empty);
            result.Setup(context => context.Request.Browser.CreateHtmlTextWriter(It.IsAny<TextWriter>())).Returns((TextWriter tw) => new HtmlTextWriter(tw));
            result.Setup(context => context.Request.Browser.EcmaScriptVersion).Returns(new Version("5.0"));
            result.Setup(context => context.Request.Browser.SupportsCss).Returns(true);
            result.Setup(context => context.Request.Browser.MajorVersion).Returns(7);
            result.Setup(context => context.Request.Browser.IsBrowser("IE")).Returns(false);
            result.Setup(context => context.Request.QueryString).Returns(new NameValueCollection());
            result.Setup(context => context.Request.Headers).Returns(new NameValueCollection { { "Accept-Encoding", "gzip" } });
            result.Setup(context => context.Items).Returns(new Hashtable());
            result.Setup(context => context.Response.Output).Returns(new Mock<TextWriter>().Object);

            result.Setup(context => context.Response.Filter).Returns(new Mock<Stream>().Object);
            // ReSharper disable AccessToStaticMemberViaDerivedType
            result.Setup(context => context.Response.ContentEncoding).Returns(UTF8Encoding.Default);
            // ReSharper restore AccessToStaticMemberViaDerivedType
            result.Setup(context => context.Response.ApplyAppPathModifier(It.IsAny<string>())).Returns<string>(r => AppPathModifier + r);
            return result;
        }
        [DebuggerStepThrough]
        public static RequestContext CreateRequestContext()
        {
            return new RequestContext(CreateMockedHttpContext().Object, new RouteData
            {
                Values =
                    {
                        {"controller", "home"},
                        {"action", "index"}
                    }
            });
        }

        [DebuggerStepThrough]
        public static void RegisterDummyRoutes(RouteCollection routes)
        {
            routes.Clear();

            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");
            routes.MapRoute("ProductList", "Products", new { controller = "Product", action = "List" });
            routes.MapRoute("ProductDetail", "Products/Detail/{id}", new { controller = "Product", action = "Detail", id = string.Empty });
            routes.MapRoute("Default", "{controller}/{action}/{id}", new { controller = "Home", action = "Index", id = string.Empty });
        }

        public static ViewContext CreateViewContext()
        {
            return new ViewContext(CreateControllerContext(), new Mock<IView>().Object, new ViewDataDictionary(), new TempDataDictionary(), TextWriter.Null);
        }

        public static ControllerContext CreateControllerContext()
        {
            return new ControllerContext(CreateRequestContext(), new ControllerTestDouble(new Dictionary<string,
                ValueProviderResult>(), new Mock<IViewDataContainer>().Object.ViewData));
        }
        [DebuggerStepThrough]
        public static void RegisterDummyRoutes()
        {
            RegisterDummyRoutes(RouteTable.Routes);
        }
    }
}